//
//  CirclePlugIn.m
//  Circle
//
//  Created by Hesam Ohadi on 6/3/11.
//  Copyright (c) 2011 Media Cast. All rights reserved.
//

/* It's highly recommended to use CGL macros instead of changing the current context for plug-ins that perform OpenGL rendering */
#import <OpenGL/CGLMacro.h>


#import "GLFilledCirclePlugIn.h"


#define	kQCPlugIn_Name				@"GL Filled Circle"
#define	kQCPlugIn_Description		@"Draws a Filled Circle with Triangle Mesh (Ported from Cinder)"

@implementation GLFilledCirclePlugIn

//@dynamic inputX, inputY, inputZ, inputWidth, inputHeight, inputSegments, inputBlendMode, inputFill, inputColor;
@dynamic inputBlendMode, inputColor, inputFill, inputSegments, inputLineWidth, inputHeight, inputWidth, inputZ, inputY, inputX;
/*
 Here you need to declare the input / output properties as dynamic as Quartz Composer will handle their implementation
 @dynamic inputFoo, outputBar;
 */

+ (NSDictionary*) attributes
{
	/*
	 Return a dictionary of attributes describing the plug-in (QCPlugInAttributeNameKey, QCPlugInAttributeDescriptionKey...).
	 */
	
	return [NSDictionary dictionaryWithObjectsAndKeys:kQCPlugIn_Name, QCPlugInAttributeNameKey, kQCPlugIn_Description, QCPlugInAttributeDescriptionKey, nil];
}

+ (NSDictionary*) attributesForPropertyPortWithKey:(NSString*)key
{
	/*
	 Specify the optional attributes for property based ports (QCPortAttributeNameKey, QCPortAttributeDefaultValueKey...).
	 */
	if([key isEqualToString:@"inputX"])
		return [NSDictionary dictionaryWithObjectsAndKeys:@"X Position", QCPortAttributeNameKey,[NSNumber numberWithDouble:0.0f], QCPortAttributeDefaultValueKey, nil];
	if([key isEqualToString:@"inputY"])
		return [NSDictionary dictionaryWithObjectsAndKeys:@"Y Position", QCPortAttributeNameKey,[NSNumber numberWithDouble:0.0f], QCPortAttributeDefaultValueKey, nil];
	if([key isEqualToString:@"inputZ"])
		return [NSDictionary dictionaryWithObjectsAndKeys:@"Z Position", QCPortAttributeNameKey,[NSNumber numberWithDouble:0.0f], QCPortAttributeDefaultValueKey, nil];
	if([key isEqualToString:@"inputColor"])
		return [NSDictionary dictionaryWithObjectsAndKeys:@"Color", QCPortAttributeNameKey, nil];
	if([key isEqualToString:@"inputFill"])
		return [NSDictionary dictionaryWithObjectsAndKeys:@"Fill", QCPortAttributeNameKey, nil];
	if([key isEqualToString:@"inputWidth"])
		return [NSDictionary dictionaryWithObjectsAndKeys:@"Width", QCPortAttributeNameKey,[NSNumber numberWithDouble:0.0f], QCPortAttributeDefaultValueKey,[NSNumber numberWithDouble:0.0f], QCPortAttributeMinimumValueKey,  nil];
	if([key isEqualToString:@"inputHeight"])
		return [NSDictionary dictionaryWithObjectsAndKeys:@"Height", QCPortAttributeNameKey,[NSNumber numberWithDouble:0.0f], QCPortAttributeDefaultValueKey,[NSNumber numberWithDouble:0.0f], QCPortAttributeMinimumValueKey,  nil];
	if([key isEqualToString:@"inputSegments"])
		return [NSDictionary dictionaryWithObjectsAndKeys:@"Segments", QCPortAttributeNameKey,[NSNumber numberWithUnsignedInt:50], QCPortAttributeDefaultValueKey,[NSNumber numberWithUnsignedInt:2], QCPortAttributeMinimumValueKey,  nil];
	if([key isEqualToString:@"inputBlendMode"])
		return [NSDictionary dictionaryWithObjectsAndKeys:@"Blending", QCPortAttributeNameKey, 
				[NSArray arrayWithObjects:@"Replace",@"Over",@"Add",@"Alpha", nil], QCPortAttributeMenuItemsKey,
				[NSNumber numberWithUnsignedInt:3], QCPortAttributeMaximumValueKey,
				[NSNumber numberWithUnsignedInt:0], QCPortAttributeDefaultValueKey,
				[NSNumber numberWithUnsignedInt:0], QCPortAttributeMinimumValueKey, nil];
	if([key isEqualToString:@"inputLineWidth"])
		return [NSDictionary dictionaryWithObjectsAndKeys:@"Line Width", QCPortAttributeNameKey,
				[NSNumber numberWithDouble:0.1f], QCPortAttributeMinimumValueKey, nil];
	return nil;
}

+ (QCPlugInExecutionMode) executionMode
{
	/*
	 Return the execution mode of the plug-in: kQCPlugInExecutionModeProvider, kQCPlugInExecutionModeProcessor, or kQCPlugInExecutionModeConsumer.
	 */
	
	return kQCPlugInExecutionModeConsumer;
}

+ (QCPlugInTimeMode) timeMode
{
	/*
	 Return the time dependency mode of the plug-in: kQCPlugInTimeModeNone, kQCPlugInTimeModeIdle or kQCPlugInTimeModeTimeBase.
	 */
	
	return kQCPlugInTimeModeNone;
}

- (id) init
{
	if(self = [super init]) {
		/*
		 Allocate any permanent resource required by the plug-in.
		 */
			}
	
	return self;
}

- (void) finalize
{
	/*
	 Release any non garbage collected resources created in -init.
	 */
	
	[super finalize];
}

- (void) dealloc
{
	/*
	 Release any resources created in -init.
	 */
	
	[super dealloc];
}

@end

@implementation GLFilledCirclePlugIn (Execution)

- (BOOL) startExecution:(id<QCPlugInContext>)context
{
	/*
	 Called by Quartz Composer when rendering of the composition starts: perform any required setup for the plug-in.
	 Return NO in case of fatal failure (this will prevent rendering of the composition to start).
	 */
	
	return YES;
}

- (void) enableExecution:(id<QCPlugInContext>)context
{
	/*
	 Called by Quartz Composer when the plug-in instance starts being used by Quartz Composer.
	 */
}


- (void) disableExecution:(id<QCPlugInContext>)context
{
	/*
	 Called by Quartz Composer when the plug-in instance stops being used by Quartz Composer.
	 */
}

- (void) stopExecution:(id<QCPlugInContext>)context
{
	/*
	 Called by Quartz Composer when rendering of the composition stops: perform any required cleanup for the plug-in.
	 */
}
- (BOOL) execute:(id<QCPlugInContext>)context atTime:(NSTimeInterval)time withArguments:(NSDictionary*)arguments
{
	CGLContextObj					cgl_ctx = [context CGLContextObj];
	const CGFloat*					colorComponents;
	GLenum							error;
	
	if(cgl_ctx == NULL)
		return NO;
	
	bool filled= self.inputFill ;
	int segments = self.inputSegments;
	GLfloat width= self.inputWidth;
	GLfloat height= self.inputHeight;
	
	GLfloat vertices[segments*2 + 2];
	
	int count=0;
	GLfloat i;
	
	for( i = 0.0f ; i < 360.0f; i+=(360.0f/segments) ) {
		vertices[count++] = (cos(i * M_PI / 180) * width);
		vertices[count++] = (sin(i * M_PI / 180) * height);
	}
	colorComponents = CGColorGetComponents(self.inputColor);
	

	/* Set current color (no need to save / restore as the current color is part of the GL_CURRENT_BIT) */
	glColor4f(colorComponents[0], colorComponents[1], colorComponents[2], colorComponents[3]);
	
	glPushMatrix();
	if(self.inputBlendMode == 0)
		glDisable(GL_BLEND);
	else if(self.inputBlendMode == 1)
	{
		glEnable(GL_BLEND);
		glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
	}
	else if(self.inputBlendMode == 2)
	{
		glEnable(GL_BLEND);
		glBlendFunc(GL_ONE, GL_ONE);
	}
	else if(self.inputBlendMode == 3)
	{
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	}
	
	glTranslatef(self.inputX,self.inputY,self.inputZ);
	
	//glVertexPointer( 2, GL_FLOAT, 0, vertices );
	//glDrawArrays( GL_TRIANGLE_FAN, 0, numSegments + 1 );
	

	
	glEnable(GL_LINE_SMOOTH);
	glEnable(GL_POLYGON_SMOOTH);
	glLineWidth(self.inputLineWidth);
	glHint(GL_LINE_SMOOTH_HINT,GL_NICEST);
	glHint(GL_POLYGON_SMOOTH_HINT,GL_NICEST);
	
	glEnableClientState( GL_VERTEX_ARRAY );
	glVertexPointer (2, GL_FLOAT , 0, vertices); 
	glDrawArrays ((filled) ? GL_TRIANGLE_FAN : GL_LINE_LOOP, 0, segments);
	glDisableClientState( GL_VERTEX_ARRAY );
	
	glPopMatrix();
	glDisable(GL_BLEND);
	glDisable(GL_LINE_SMOOTH);
	glDisable(GL_POLYGON_SMOOTH);
	
	/* Check for OpenGL errors */
	if(error = glGetError())
		[context logMessage:@"OpenGL error %04X", error];
	
	return (error ? NO : YES);
	
}

@end