//
//  QCOpenGLPort_Blending.m
//  GLTools
//
//  Created by Christopher Wright on 10/8/08.
//  Copyright 2008 Kosada Incorporated. All rights reserved.
//

#import <OpenGL/CGLMacro.h>
#import "QCOpenGLContext.h"
#import "QCOpenGLPort_Blending-kineme.h"

@implementation QCOpenGLPort_Blending (kineme)

- (id)initWithNode:(id)fp8 arguments:(id)fp12
{
	if(self = [super initWithNode: fp8 arguments: [self _argumentsFromAttributesKey:@"_blendingPortAttributes" originalArguments:fp12]])
		[self setMaxIndexValue: 3];	// 2 by default, plus each additional mode added in attributes
	return self;
}

-(NSDictionary*)attributes
{
	NSMutableDictionary *attributes = [[[super attributes] mutableCopy] autorelease];
	[[attributes objectForKey: @"menu"] addObject: @"Alpha"];
	
	//glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	
	return attributes;
}

- (void)setOnOpenGLContext:(QCOpenGLContext*)context
{
	GLint sfunc, dfunc;
	unsigned int mode;
	
	CGLContextObj cgl_ctx = [context CGLContextObj];
	
#ifdef __LP64__
	unsigned char KinemeBlendingEnabled;
	int KinemeBlendingSourceFunc, KinemeBlendingDestFunc;
	glGetIntegerv(GL_BLEND_SRC,&KinemeBlendingSourceFunc);
	glGetIntegerv(GL_BLEND_DST,&KinemeBlendingDestFunc);
	KinemeBlendingEnabled = glIsEnabled(GL_BLEND);
#else
	glGetIntegerv(GL_BLEND_SRC,&_sourceFunction);
	glGetIntegerv(GL_BLEND_DST,&_destFunction);
	_enabled = glIsEnabled(GL_BLEND);
#endif
	
	mode = [self indexValue];
	
	if(mode)
		glEnable(GL_BLEND);
	else
		glDisable(GL_BLEND);
	
	switch(mode)
	{
		case 0: // replace
			sfunc = dfunc = GL_ZERO;
			break;
		case 1:	// over
			sfunc = GL_ONE;
			dfunc = GL_ONE_MINUS_SRC_ALPHA;
			break;
		case 2:	// add
			sfunc = dfunc = GL_ONE;
			break;
		case 3:	// Alpha!
			sfunc = GL_SRC_ALPHA;
			dfunc = GL_ONE_MINUS_SRC_ALPHA;
	}
	// save a state change if we don't need it...
#ifdef __LP64__
	object_setInstanceVariable(self, "_enabled", KinemeBlendingEnabled);
	object_setInstanceVariable(self, "_destFunction", KinemeBlendingDestFunc);
	object_setInstanceVariable(self, "_sourceFunction", KinemeBlendingSourceFunc);
	
	if(sfunc != KinemeBlendingSourceFunc || dfunc != KinemeBlendingDestFunc)
		glBlendFunc(sfunc, dfunc);
#else
	if(sfunc != _sourceFunction || dfunc != _destFunction)
		glBlendFunc(sfunc, dfunc);
#endif
}

- (void)unsetOnOpenGLContext:(QCOpenGLContext*)context
{
	CGLContextObj cgl_ctx = [context CGLContextObj];
#ifdef __LP64__
	unsigned int KinemeBlendingEnabled;
	unsigned int KinemeBlendingSourceFunc, KinemeBlendingDestFunc;
	// in 64-bit mode, QCOpenGLPort_Blending's ivars aren't exported(hidden/private), so linking fails :(
	object_getInstanceVariable(self, "_destFunction", (void**)&KinemeBlendingDestFunc);
	object_getInstanceVariable(self, "_sourceFunction", (void**)&KinemeBlendingSourceFunc);
	object_getInstanceVariable(self, "_enabled", (void**)&KinemeBlendingEnabled);
	glBlendFunc(KinemeBlendingSourceFunc, KinemeBlendingDestFunc);
	if(!KinemeBlendingEnabled)
		glDisable(GL_BLEND);
#else
	glBlendFunc(_sourceFunction, _destFunction);
	if(!_enabled)
		glDisable(GL_BLEND);
#endif
}

@end
